from rich import print
from rich.panel import Panel
from rich.text import Text
from rich.table import Table
table = Table(title="", box=None)
table.add_column("server", justify="left", style="cyan", no_wrap=True)
table.add_column("in-net", justify="left", style="#df412f")
table.add_column("out-net", justify="left", style="green")
table.add_column("out-net", justify="left", style="#f5bb00")

table.add_row("tft", "http://localhost:8800", "http://localhost:58800", "./TFT-project/tft.log")
table.add_row("streamlit", "http://localhost:8501", "http://localhost:34501", "./streamlit.log")
table.add_row("callback", "http://localhost:33221", "http://localhost:33221", "./callback.log")

print(Panel(table, title="🤗MVTF is ready to serve!", expand=False, ))

text = Text("Hello\n asdf\nasldfj\nasdf", justify="center")


def _init_table(n_cols=3):
    table = Table(
        title="", box=None, highlight=True, show_header=False, min_width=50
    )
    table.add_column('', justify='left')
    for _ in range(n_cols-1):
        table.add_column('', justify='right')
    return table


ttable = _init_table(n_cols=2)



# print(Panel(table, title="🤗MVTF is ready to serve!", expand=False))



import pretty_errors
# from .widgets import timer


class Cli:
    def __init__(self):
        ...

    @staticmethod
    def save_docker_images(filedir='.', skip_exists=True, use_stream=False):
        kwargs = locals()
        from .docker import save_docker_images
        return save_docker_images(**kwargs)

    @staticmethod
    def load_docker_images(filename_pattern="./*", skip_exists=True):
        kwargs = locals()
        from .docker import load_docker_images
        return load_docker_images(**kwargs)

    @staticmethod
    def docker_gpu_stat():
        from .docker.nvidia_stat import docker_gpu_stat
        return docker_gpu_stat()

    @staticmethod
    def pack(source_path: str, target_path=None, format='gztar'):
        kwargs = locals()
        from .utils.compress import pack
        return pack(**kwargs)

    @staticmethod
    def unpack(filename: str, extract_dir=None, format=None):
        kwargs = locals()
        from .utils.compress import unpack
        return unpack(**kwargs)

    @staticmethod
    def start_server(port=50001, deque_maxlen=None):
        kwargs = locals()
        from .multiprocess import start_server
        return start_server(**kwargs)

    @staticmethod
    def kill(*ports: int, view=False):
        from .multiprocess import kill
        return kill(*ports, view)

    @staticmethod
    def clone(url: str, repo_name=None, branch=None):
        kwargs = locals()
        from .cli import clone
        return clone(**kwargs)

    @staticmethod
    def get_ip(env="inner"):
        kwargs = locals()
        from .utils.net import get_ip
        return get_ip(**kwargs)

    @staticmethod
    def create(project_name: str, out=None):
        """创建项目
        Parameter
        ---------
        project_name : str
            package name
        out : str | None
            项目生成路径
        """
        if out is None:
            out = project_name
        from .template.scaffold.core import create_project
        return create_project(project_name, out)

    @staticmethod
    def milvus(flag='start'):
        kwargs = locals()
        from .ann import milvus
        return milvus(**kwargs)


def fire_commands():
    import fire
    fire.Fire(Cli)


def typer_commands():
    import typer
    app = typer.Typer()
    # [app.command()(i) for i in func_list]
    # app()


def main():
    use_fire = 1
    if use_fire:
        fire_commands()
    else:
        # Fixme *形参 传入会出错，参考这里 https://typer.tiangolo.com/tutorial/multiple-values/arguments-with-multiple-values/
        typer_commands()



# 前置条件 服务端上已安装 systemd, 安装示例 `apt install systemd`
from sparrow.nat_traversal.install import Installer
from sparrow.nat_traversal.server_config_template import *
import os


class Manager:
    def __init__(self):
        self.installer = Installer()

    def install_frp(self):
        self.installer.install()

    def write_systemd(self):
        write_frps_to_systemd()



def write_frps_to_systemd():
    # vim /etc/systemd/system/frps.service
    installer = Installer()
    frps_path = os.path.join(installer.frp_unpack_file_dir, "frps")
    frps_ini_path = os.path.join(installer.frp_unpack_file_dir, "frps.ini")
    content = f"""\
[Unit]
# 服务名称，可自定义
Description = frp server
After = network.target syslog.target
Wants = network.target

[Service]
Type = simple
# 启动frps的命令，需修改为您的frps的安装路径
ExecStart = {frps_path} -c {frps_ini_path}

[Install]
WantedBy = multi-user.target
"""
    frp_systemd_file = "/etc/systemd/system/frps.service"
    try:
        with open(frp_systemd_file, 'w') as f:
            f.write(content)
    except Exception as e:
        print(e)
        print(f"Write to {frp_systemd_file} failed! Please write the following content into it manually.")
        print(content)


def start_server():
    os.system("systemctl start frps")


def stop_server():
    os.system("systemctl stop frps")


def restart_server():
    os.system("systemctl restart frps")


def enable_server():
    """开机自启"""
    os.system("systemctl enable frps")



import time
import psutil

_scale = {'KB': 1024, 'MB': 1024 * 1024, 'GB': 1024 * 1024 * 1024, }


def get_virtual_memory(unit='GB'):
    memory_tuple = psutil.virtual_memory()
    _key = unit.upper()
    memory_dict = {
        "total": (memory_tuple[0] / _scale[_key], _key),
        "available": (memory_tuple[1] / _scale[_key], _key),
        "percent": (memory_tuple[2], '%'),
        "used": (memory_tuple[3] / _scale[_key], _key),
        "free": (memory_tuple[4] / _scale[_key], _key),
    }
    return memory_dict


def elapsed_since(start):
    return time.strftime("%H:%M:%S", time.gmtime(time.time() - start))


def get_process_memory(unit='MB'):
    return psutil.Process().memory_info().rss / _scale[unit.upper()]


def _profile(func):
    """
    注意，这个装饰器只能得到函数推出后的内存增量，而要函数内的内存增量不应用此装饰器
    若要方便调试对函数的使用，请使用 memory_profiler这个包
    from memory_profiler import profile
    @profile
    def func():
        ...
    """

    def wrapper(*args, **kwargs):
        _key = "MB"
        mem_before = get_process_memory(_key)
        start = time.time()
        result = func(*args, **kwargs)
        elapsed_time = elapsed_since(start)
        mem_after = get_process_memory(_key)
        print(
            f"{func.__name__}: memory before: {mem_before:.2f} {_key}, after: {mem_after:.2f} {_key}, consumed: {mem_after - mem_before:.2f} {_key}; exec time: {elapsed_time}"
        )
        return result

    return wrapper


if __name__ == "__main__":
    from memory_profiler import profile

    @profile
    def test1():
        import numpy as np
        _scale = {'kB': 1024.0, 'mB': 1024.0 * 1024.0, 'KB': 1024.0, 'MB': 1024.0 * 1024.0}
        _key = "MB"
        mem_before = get_process_memory()
        start = time.time()
        a = np.linspace(0, 100, 100000)
        b = np.linspace(0, 100, 100000)
        c = np.random.rand(100, 100, 1000)
        elapsed_time = elapsed_since(start)
        mem_after = get_process_memory()
        print(
            f" memory before: {mem_before:.2f}{_key}, after: {mem_after:.2f}{_key}, consumed: {mem_after - mem_before:.2f}{_key}; exec time: {elapsed_time}"
        )


    test1()
    print(get_virtual_memory())




def gen_frpc_ini():
    content = """\
[common]
server_addr = x.x.x.x
server_port = 7000

[web]
type = http
local_port = 80
custom_domains = www.yourdomain.com

[web2]
type = http
local_port = 8080
custom_domains = www.yourdomain2.com
"""


from pathlib import Path
from typing import Union, List
import inspect
from glob import glob as _glob
import os
import sys


def rel_to_abs(rel_path: Union[str, Path], parents=0, return_str=True, strict=False):
    """Return absolute path relative to the called file
    args:
        parent: <int> The number of times `f_back` will be calledd.
    """
    currentframe = inspect.currentframe()
    f = currentframe.f_back
    for _ in range(parents):
        f = f.f_back
    current_path = Path(f.f_code.co_filename).parent
    pathlib_path = current_path / rel_path
    pathlib_path = pathlib_path.resolve(strict=strict)
    if return_str:
        return str(pathlib_path)
    else:
        return pathlib_path


def rel_path_join(*paths: Union[str, Path], return_str=True):
    return rel_to_abs(os.path.join(*paths), parents=1, return_str=return_str)


def ls(_dir, *patterns, relp=True, concat='extend', recursive=False):
    """
    Example:
    --------
        >>> ls("./data/", "*.jpg", "*.png")
    """
    if relp:
        _dir = rel_to_abs(_dir, parents=1, return_str=True, strict=False)
    path_list = []
    for pattern in patterns:
        if concat == 'extend':
            path_list.extend(_glob(os.path.join(_dir, pattern), recursive=recursive))
        else:
            path_list.append(_glob(os.path.join(_dir, pattern), recursive=recursive))
    return path_list


def add_env_path(rel_paths: Union[str, List[str]] = '.'):
    """
    Example:
    --------
        >>> add_env_path('..')
        >>> add_env_path('..', '../..')
    """

    if isinstance(rel_paths, str):
        rel_paths = [rel_paths]
    [sys.path.append(rel_to_abs(i, parents=1)) for i in rel_paths]



import os
import time

from sparrow import rel_to_abs, rm
import requests
from tqdm import tqdm
import sys


class Installer:
    proxy_prefix = "https://ghproxy.com/"
    frp_dir = rel_to_abs('./frp/', return_str=True)

    def __init__(self, frp_version="0.44.0", platform='linux', framework='amd64', china_source=True, use_httpie=True):
        """
        platform: "linux", "darwin", "windows", "freebsd"
        framework: "amd64","arm", "arm64", "386", "mips64"
        """
        file_suffix = "zip" if platform == "windows" else "tar.gz"
        frp_release_name = f"frp_{frp_version}_{platform}_{framework}.{file_suffix}"
        frp_unpack_name = f"frp_{frp_version}_{platform}_{framework}"
        self.frp_release_name = frp_release_name
        self.frp_release_file_path = os.path.join(self.frp_dir, self.frp_release_name)
        self.frp_unpack_file_dir = os.path.join(self.frp_dir, frp_unpack_name)
        self.frp_download_url = f"https://github.com/fatedier/frp/releases/download/v{frp_version}/{frp_release_name}"
        self._china_source = china_source
        self._use_httpie = use_httpie

    def clean_dir(self):
        rm(self.frp_dir)
        rm(self.frp_release_file_path)

    def install(self):
        self.download_frp()
        time.sleep(0.1)
        self.unpack()

    def unpack(self):
        os.system(f"sparrow unpack {self.frp_release_file_path} {self.frp_dir}")

    def download_frp(self):
        if not os.path.exists(self.frp_dir):
            os.mkdir(self.frp_dir)
        url = f"{self.proxy_prefix}{self.frp_download_url}" if self._china_source else self.frp_download_url
        # print(self.frp_release_file_path)
        if self._use_httpie:
            os.system(f"http -d {url} -o {self.frp_release_file_path}")
        else:
            response = requests.get(url, stream=True)
            total = response.headers.get('content-length')
            with open(self.frp_release_file_path, 'wb') as f:
                if total is None:
                    for chunk in tqdm(response.iter_content(chunk_size=1024*500)):
                        f.write(chunk)
                else:
                    downloaded = 0
                    total = int(total)
                    print(f"Downloading {self.frp_release_name} of size {total/1024/1024:.2f} Mb...")
                    for chunk in response.iter_content(chunk_size=max(int(total/1000), 1024*1024)):
                        downloaded += len(chunk)
                        f.write(chunk)
                        done = int(50 * downloaded / total)
                        sys.stdout.write('\r[{}{}]'.format('█' * done, '.' * (50 - done)))
                        sys.stdout.flush()
                    print('\nDone.')


if __name__ == "__main__":
    ins = Installer()
    ins.clean_dir()
    ins.install()
    # ins.unpack()
